@@ -12,20 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""#### Module for an indexable circular array data structure."""
-
+"""
+### Module for an indexable circular array data structure.
+"""
 from __future__ import annotations
-from typing import Callable, cast, Generic, Iterator, Optional, TypeVar
+from typing import Callable, cast, Generic, Iterator, Optional, TypeVar, Never
 
 __all__ = ['CA']
 
 D = TypeVar('D')
-T = TypeVar('T')
+U = TypeVar('U')
 L = TypeVar('L')
 R = TypeVar('R')
 
 class CA(Generic[D]):
-    """#### Class implementing an indexable circular array data structure.
+    """
+    #### Indexable circular array data structure
 
     * generic, stateful data structure
     * amortized O(1) pushing and popping from either end
@@ -34,8 +36,8 @@ class CA(Generic[D]):
     * makes defensive copies of contents for the purposes of iteration
     * not sliceable
     * in boolean context returns true if not empty, false if empty
-    * in comparisons will compare contents with identity before equality
-      * like Python builtins like tuples, lists, and dicts do
+    * in comparisons contents compare with identity before equality
+      * as do Python tuples, lists, and dicts
     * raises `IndexError` for out-of-bounds indexing
     * raises `ValueError` for popping from or folding an empty CA
     """
@@ -52,25 +54,33 @@ class CA(Generic[D]):
             self._front = 1
             self._rear = capacity - 2
 
+    def _double_storage_capacity(self) -> None:
+        if self._front <= self._rear:
+            self._list += [None]*self._capacity
+            self._capacity *= 2
+        else:
+            self._list = self._list[:self._front] + [None]*self._capacity + self._list[self._front:]
+            self._front, self._capacity = self._front + self._capacity, 2*self._capacity
+
     def __iter__(self) -> Iterator[D]:
         if self._count > 0:
-            capacity,       rear,       position,    currentState = \
-            self._capacity, self._rear, self._front, self._list.copy()
+            capacity, rear, position, current_state = \
+                self._capacity, self._rear, self._front, self._list.copy()
 
             while position != rear:
-                yield cast(D, currentState[position])  # will always yield a D
+                yield cast(D, current_state[position])
                 position = (position + 1) % capacity
-            yield cast(D, currentState[position])  # will always yield a D
+            yield cast(D, current_state[position])
 
     def __reversed__(self) -> Iterator[D]:
         if self._count > 0:
-            capacity,       front,       position,   currentState = \
-            self._capacity, self._front, self._rear, self._list.copy()
+            capacity, front, position, current_state = \
+                self._capacity, self._front, self._rear, self._list.copy()
 
             while position != front:
-                yield cast(D, currentState[position])  # will always yield a D
+                yield cast(D, current_state[position])
                 position = (position - 1) % capacity
-            yield cast(D, currentState[position])  # will always yield a D
+            yield cast(D, current_state[position])
 
     def __repr__(self) -> str:
         return 'CA(' + ', '.join(map(repr, self)) + ')'
@@ -87,11 +97,9 @@ class CA(Generic[D]):
     def __getitem__(self, index: int) -> D:
         cnt = self._count
         if 0 <= index < cnt:
-            return cast(D, self._list[(self._front + index)
-                                       % self._capacity])  # will always return a D
+            return cast(D, self._list[(self._front + index) % self._capacity])
         elif -cnt <= index < 0:
-            return cast(D, self._list[(self._front + cnt + index)
-                                       % self._capacity])  # will always return a D
+            return cast(D, self._list[(self._front + cnt + index) % self._capacity])
         else:
             if cnt > 0:
                 msg1 = 'Out of bounds: '
@@ -124,8 +132,12 @@ class CA(Generic[D]):
         if not isinstance(other, type(self)):
             return False
 
-        frontL,      capacityL,      countL,      frontR,       capacityR,       countR = \
-        self._front, self._capacity, self._count, other._front, other._capacity, other._count
+        frontL, frontR, \
+        countL, countR, \
+        capacityL, capacityR = \
+            self._front, other._front, \
+            self._count, other._count, \
+            self._capacity, other._capacity
 
         if countL != countR:
             return False
@@ -138,67 +150,58 @@ class CA(Generic[D]):
         return True
 
     def pushL(self, *ds: D) -> None:
-        """##### Push data from the left onto the CircularArray."""
+        """Push data from the left onto the CA."""
         for d in ds:
             if self._count == self._capacity:
-                self.double()
+                self._double_storage_capacity()
             self._front = (self._front - 1) % self._capacity
-            self._list[self._front] = d
-            self._count += 1
+            self._list[self._front], self._count = d, self._count + 1
 
     def pushR(self, *ds: D) -> None:
-        """##### Push data from the right onto the CircularArray."""
+        """Push data from the right onto the CA."""
         for d in ds:
             if self._count == self._capacity:
-                self.double()
+                self._double_storage_capacity()
             self._rear = (self._rear + 1) % self._capacity
-            self._list[self._rear] = d
-            self._count += 1
+            self._list[self._rear], self._count = d, self._count + 1
 
-    def popL(self) -> D:
-        """##### Pop one value off the left side of the CircularArray.
+    def popL(self) -> D|Never:
+        """Pop one value off the left side of the CA.
 
-            * raises `ValueError` when called on an empty CA"""
-        if self._count > 0:
-            d, \
-            self._list[self._front], \
-            self._front, \
-            self._count \
-                = \
-            self._list[self._front], \
-            None, \
-            (self._front+1) % self._capacity, \
-            self._count-1
-            return cast(D, d)  # will always yield a D
-        else:
+        * raises `ValueError` when called on an empty CA
+        """
+        if self._count > 1:
+            d, self._list[self._front], self._front, self._count = \
+                self._list[self._front], None, (self._front+1) % self._capacity, self._count - 1
+        elif self._count < 1:
             msg = 'Method popL called on an empty CA'
             raise ValueError(msg)
+        else:
+            d, self._list[self._front], self._count, self._front, self._rear = \
+                self._list[self._front], None, 0, 0, self._capacity - 1
+        return cast(D, d)
 
-    def popR(self) -> D:
-        """##### Pop one value off the right side of the CircularArray.
+    def popR(self) -> D|Never:
+        """Pop one value off the right side of the CA.
 
-            * raises `ValueError` when called on an empty CA
+        * raises `ValueError` when called on an empty CA
         """
         if self._count > 0:
-            d, \
-            self._list[self._rear], \
-            self._rear, \
-            self._count \
-                = \
-            self._list[self._rear], \
-            None, \
-            (self._rear - 1) % self._capacity, \
-            self._count-1
-            return cast(D, d)  # will always yield a D
-        else:
+            d, self._list[self._rear], self._rear, self._count = \
+                self._list[self._rear], None, (self._rear - 1) % self._capacity, self._count - 1
+        elif self._count < 1:
             msg = 'Method popR called on an empty CA'
             raise ValueError(msg)
+        else:
+            d, self._list[self._front], self._count, self._front, self._rear = \
+                self._list[self._front], None, 0, 0, self._capacity - 1
+        return cast(D, d)
 
     def popLD(self, default: D) -> D:
-        """##### Pop one value from left, provide a mandatory default value.
+        """Pop one value from left, provide a mandatory default value.
 
-            * safe version of popL
-            * returns a default value in the event the `CA` is empty
+        * safe version of popL
+        * returns a default value in the event the `CA` is empty
         """
         try:
             return self.popL()
@@ -206,10 +209,10 @@ class CA(Generic[D]):
             return default
 
     def popRD(self, default: D) -> D:
-        """##### Pop one value from right, provide a mandatory default value.
+        """Pop one value from right, provide a mandatory default value.
 
-            * safe version of popR
-            * returns a default value in the event the `CA` is empty
+        * safe version of popR
+        * returns a default value in the event the `CA` is empty
         """
         try:
             return self.popR()
@@ -217,12 +220,12 @@ class CA(Generic[D]):
             return default
 
     def popLT(self, max: int=1) -> tuple[D, ...]:
-        """##### Pop multiple values from left side of CircularArray
+        """Pop multiple values from left side of CA.
 
-            * returns the results in a `tuple` of type `tuple[~D, ...]`
-            * returns an empty tuple if `CA` is empty
-            * pop no more that `max` values
-            * will pop less if `CA` becomes empty
+        * returns the results in a `tuple` of type `tuple[~D, ...]`
+        * returns an empty tuple if `CA` is empty
+        * pop no more that `max` values
+        * will pop less if `CA` becomes empty
         """
         ds: list[D] = []
 
@@ -237,12 +240,12 @@ class CA(Generic[D]):
         return tuple(ds)
 
     def popRT(self, max: int=1) -> tuple[D, ...]:
-        """##### Pop multiple values from right side of CircularArray
+        """Pop multiple values from right side of CA.
 
-            * returns the results in a `tuple` of type `tuple[~D, ...]`
-            * returns an empty tuple if `CA` is empty
-            * pop no more that `max` values
-            * will pop less if `CA` becomes empty
+        * returns the results in a `tuple` of type `tuple[~D, ...]`
+        * returns an empty tuple if `CA` is empty
+        * pop no more that `max` values
+        * will pop less if `CA` becomes empty
         """
         ds: list[D] = []
         while max > 0:
@@ -255,25 +258,24 @@ class CA(Generic[D]):
 
         return tuple(ds)
 
-    def map(self, f: Callable[[D], T]) -> CA[T]:
-        """##### Apply function f over contents, returns new CircularArray instance.
+    def map(self, f: Callable[[D], U]) -> CA[U]:
+        """Apply function f over contents, returns new CA instance.
 
-            * parameter `f` generic function of type `f[~D, ~T] -> CA[~T]`
-            * returns a new instance of type `CA[~T]``
+        * parameter `f` function of type `f[~D, ~U] -> CA[~U]`
+        * returns a new instance of type `CA[~U]``
         """
         return CA(*map(f, self))
 
     def foldL(self, f: Callable[[L, D], L], initial: Optional[L]=None) -> L:
-        """##### Left fold CircularArray via function and optional initial value.
+        """Left fold CA via function and optional initial value.
 
-            * parameter `f` generic function of type `f[~L, ~D] -> ~L`
-              * the first argument to `f` is for the accumulated value.
-            * parameter `initial` is an optional initial value
-              * note that if not given then it will be the case that `~L = ~D`
-            * returns the reduced value of type `~L`
-              * note that `~L` and `~D` can be the same type
-              * if an initial value is not given then by necessity `~L = ~D` 
-            * raises `ValueError` when called on an empty `CA` and `initial` not given
+        * parameter `f` function of type `f[~L, ~D] -> ~L`
+          * the first argument to `f` is for the accumulated value.
+        * parameter `initial` is an optional initial value
+        * returns the reduced value of type `~L`
+          * note that `~L` and `~D` can be the same type
+          * if an initial value is not given then by necessity `~L = ~D` 
+        * raises `ValueError` when called on an empty `CA` and `initial` not given
         """
         if self._count == 0:
             if initial is None:
@@ -294,16 +296,15 @@ class CA(Generic[D]):
                 return acc
 
     def foldR(self, f: Callable[[D, R], R], initial: Optional[R]=None) -> R:
-        """##### Right fold CircularArray via function and optional initial value.
+        """Right fold CA via function and optional initial value.
 
-            * generic function `f` of type `f[~D, ~R] -> ~R`
-              * the second argument to f is for the accumulated value
-            * parameter `initial` is an optional initial value
-              * note that if not given then it will be the case that `~R = ~D`
-            * returns the reduced value of type `~R`
-              * note that `~R` and `~D` can be the same type
-              * if `initial` is not given then by necessity `~R = ~D`
-            * raises `ValueError` when called on an empty `CA` and `initial` not given
+        * parameter `f` function of type `f[~D, ~R] -> ~R`
+          * the second argument to f is for the accumulated value
+        * parameter `initial` is an optional initial value
+        * returns the reduced value of type `~R`
+          * note that `~R` and `~D` can be the same type
+          * if an initial value is not given then by necessity `~R = ~D`
+        * raises `ValueError` when called on an empty `CA` and `initial` not given
         """
         if self._count == 0:
             if initial is None:
@@ -324,52 +325,38 @@ class CA(Generic[D]):
                 return acc
 
     def capacity(self) -> int:
-        """##### Returns current capacity of the CircularArray."""
+        """Returns current capacity of the CA."""
         return self._capacity
 
     def compact(self) -> None:
-        """##### Compact the CircularArray."""
+        """Compact the CA."""
         match self._count:
             case 0:
-                self._capacity, self._front, self._rear, self._list = \
-                2,              0,           1,          [None, None]
+                self._capacity, self._front, self._rear, self._list = 2, 0, 1, [None, None]
             case 1:
-                self._capacity, self._front, self._rear, self._list = \
-                3,              1,           1,          [None, self._list[self._front], None]
+                self._capacity, self._front, self._rear, self._list = 3, 1, 1, [None, self._list[self._front], None]
             case _:
                 if self._front <= self._rear:
-                    self._capacity, self._front, self._rear,  self._list = \
-                    self._count+2,  1,           self._count, \
-                    [None] + self._list[self._front:self._rear+1] + [None]
+                    self._capacity, self._front, self._rear, self._list = \
+                        self._count+2, 1, self._count, [None] + self._list[self._front:self._rear+1] + [None]
                 else:
-                    self._capacity, self._front, self._rear,  self._list = \
-                    self._count+2,  1,           self._count, [None] \
-                        + self._list[self._front:] + self._list[:self._rear+1] \
-                        + [None]
-
-    def double(self) -> None:
-        """##### Double the capacity of the CircularArray."""
-        if self._front <= self._rear:
-            self._list += [None]*self._capacity
-            self._capacity *= 2
-        else:
-            self._list = self._list[:self._front] + [None]*self._capacity + self._list[self._front:]
-            self._front += self._capacity
-            self._capacity *= 2
+                    self._capacity, self._front, self._rear, self._list = \
+                        self._count+2, 1, self._count, [None] + self._list[self._front:] + self._list[:self._rear+1] + [None]
 
     def empty(self) -> None:
-        """##### Empty the CircularArray, keep current capacity."""
+        """Empty the CA, keep current capacity."""
         self._list, self._front, self._rear = [None]*self._capacity, 0, self._capacity-1
 
     def fractionFilled(self) -> float:
-        """##### Returns fractional capacity of the CircularArray."""
+        """Returns fractional capacity of the CA."""
         return self._count/self._capacity
 
-    def resize(self, newSize: int= 0) -> None:
-        """##### Compact CircularArray and resize to newSize if less than newSize."""
+    def resize(self, new_capacity: int=2) -> None:
+        """Compact `CA` and resize to `new_capacity` if necessary."""
         self.compact()
-        capacity = self._capacity
-        if newSize > capacity:
-            self._list, self._capacity = self._list+[None]*(newSize-capacity), newSize
+        if new_capacity > self._capacity:
+            self._capacity, self._list = \
+                new_capacity, self._list + [None]*(new_capacity-self._capacity)
             if self._count == 0:
-                self._rear = capacity - 1
+                self._front = 0
+                self._rear = self._capacity - 1
